=== modified file 'click/cappinfo/cappinfo.go'
--- click/cappinfo/cappinfo.go	2014-07-08 23:23:13 +0000
+++ click/cappinfo/cappinfo.go	2014-09-17 19:38:34 +0000
@@ -37,6 +37,26 @@
    g_free (desktop_id);
    return filename;
 }
+
+gchar* app_symbolic_icon_from_desktop_id (gchar* desktop_id) {
+    gchar* x_symbolic_icon;
+    GIcon* symbolic_icon;
+    GDesktopAppInfo* app_info = g_desktop_app_info_new (desktop_id);
+    if (app_info != NULL) {
+        if((x_symbolic_icon = g_desktop_app_info_get_string(app_info, "X-Ubuntu-SymbolicIcon"))) {
+            GFile *file;
+            file = g_file_new_for_path(x_symbolic_icon);
+            symbolic_icon = g_file_icon_new (file);
+            g_object_unref (file);
+            g_free(x_symbolic_icon);
+            g_object_unref (app_info);
+            return g_icon_to_string(symbolic_icon);
+        }
+        g_object_unref (app_info);
+    }
+    g_free (desktop_id);
+    return NULL;
+}
 */
 import "C"
 
@@ -45,3 +65,14 @@
 	defer C.g_free((C.gpointer)(name))
 	return C.GoString((*C.char)(name))
 }
+
+func appSymbolicIconFromDesktopId(desktopId string) string {
+	name := C.app_symbolic_icon_from_desktop_id((*C.gchar)(C.CString(desktopId)))
+	if name == nil {
+		return ""
+	}
+	defer C.g_free((C.gpointer)(name))
+	return C.GoString((*C.char)(name))
+}
+
+var AppSymbolicIconFromDesktopId = appSymbolicIconFromDesktopId

=== modified file 'click/click.go'
--- click/click.go	2014-08-15 10:32:51 +0000
+++ click/click.go	2014-09-17 19:38:34 +0000
@@ -146,6 +146,10 @@
 var symbolic = _symbolic
 
 func (app *AppId) SymbolicIcon() string {
+	symbolicIcon := cappinfo.AppSymbolicIconFromDesktopId(app.DesktopId())
+	if symbolicIcon != "" {
+		return symbolicIcon
+	}
 	return symbolic(app.Icon())
 }
 

=== modified file 'click/click_test.go'
--- click/click_test.go	2014-08-15 10:32:51 +0000
+++ click/click_test.go	2014-09-17 19:38:34 +0000
@@ -22,6 +22,8 @@
 	"testing"
 
 	. "launchpad.net/gocheck"
+
+	"launchpad.net/ubuntu-push/click/cappinfo"
 )
 
 func TestClick(t *testing.T) { TestingT(t) }
@@ -200,3 +202,15 @@
 	c.Assert(err, IsNil)
 	c.Check(app.SymbolicIcon(), Equals, "xyzzy")
 }
+
+func (s *clickSuite) TestSymbolicFromDesktopFile(c *C) {
+	orig := cappinfo.AppSymbolicIconFromDesktopId
+	cappinfo.AppSymbolicIconFromDesktopId = func(desktopId string) string {
+		return "/foo/symbolic"
+	}
+	defer func() {
+		cappinfo.AppSymbolicIconFromDesktopId = orig
+	}()
+	app, _ := ParseAppId("com.ubuntu.clock_clock_1.2")
+	c.Check(app.SymbolicIcon(), Equals, "/foo/symbolic")
+}

=== modified file 'client/client.go'
--- client/client.go	2014-09-05 10:47:29 +0000
+++ client/client.go	2014-10-08 20:35:24 +0000
@@ -471,6 +471,7 @@
 // changing) the u1 account used to auth
 func (client *PushClient) handleAccountsChange() {
 	client.log.Infof("U1 account changed; restarting session")
+	client.session.ClearCookie()
 	client.session.Close()
 }
 

=== modified file 'client/service/common.go'
--- client/service/common.go	2014-08-06 02:48:10 +0000
+++ client/service/common.go	2014-10-23 14:15:18 +0000
@@ -63,7 +63,7 @@
 }
 
 // Start() dials the bus, grab the name, and listens for method calls.
-func (svc *DBusService) Start(dispatchMap bus.DispatchMap, busAddr bus.Address) error {
+func (svc *DBusService) Start(dispatchMap bus.DispatchMap, busAddr bus.Address, init func() error) error {
 	svc.lock.Lock()
 	defer svc.lock.Unlock()
 	if svc.state != StateUnknown {
@@ -89,6 +89,11 @@
 			}
 		}
 	}()
+	if init != nil {
+		if err = init(); err != nil {
+			return err
+		}
+	}
 	svc.Bus.WatchMethod(dispatchMap, "/*", svc)
 	svc.state = StateRunning
 	return nil

=== modified file 'client/service/postal.go'
--- client/service/postal.go	2014-09-09 20:46:45 +0000
+++ client/service/postal.go	2014-10-23 14:15:18 +0000
@@ -141,16 +141,16 @@
 
 // Start() dials the bus, grab the name, and listens for method calls.
 func (svc *PostalService) Start() error {
-	err := svc.DBusService.Start(bus.DispatchMap{
+	return svc.DBusService.Start(bus.DispatchMap{
 		"PopAll":          svc.popAll,
 		"Post":            svc.post,
 		"ListPersistent":  svc.listPersistent,
 		"ClearPersistent": svc.clearPersistent,
 		"SetCounter":      svc.setCounter,
-	}, PostalServiceBusAddress)
-	if err != nil {
-		return err
-	}
+	}, PostalServiceBusAddress, svc.init)
+}
+
+func (svc *PostalService) init() error {
 	actionsCh, err := svc.takeTheBus()
 	if err != nil {
 		return err

=== modified file 'client/service/service.go'
--- client/service/service.go	2014-07-14 19:51:43 +0000
+++ client/service/service.go	2014-10-23 14:15:18 +0000
@@ -89,7 +89,7 @@
 	return svc.DBusService.Start(bus.DispatchMap{
 		"Register":   svc.register,
 		"Unregister": svc.unregister,
-	}, PushServiceBusAddress)
+	}, PushServiceBusAddress, nil)
 }
 
 var (

=== modified file 'client/session/session.go'
--- client/session/session.go	2014-08-26 16:05:28 +0000
+++ client/session/session.go	2014-10-08 20:35:24 +0000
@@ -247,6 +247,12 @@
 	return sess.cookie
 }
 
+func (sess *ClientSession) ClearCookie() {
+	sess.connLock.Lock()
+	defer sess.connLock.Unlock()
+	sess.cookie = ""
+}
+
 // getHosts sets deliveryHosts possibly querying a remote endpoint
 func (sess *ClientSession) getHosts() error {
 	if sess.getHost != nil {

=== modified file 'client/session/session_test.go'
--- client/session/session_test.go	2014-09-01 16:03:06 +0000
+++ client/session/session_test.go	2014-10-08 20:35:24 +0000
@@ -1687,3 +1687,17 @@
 	// and redialJitter got called every time shouldDelay was true
 	c.Check(n, Equals, 4)
 }
+
+/****************************************************************
+  ClearCookie() tests
+****************************************************************/
+
+func (cs *clientSessionSuite) TestClearCookie(c *C) {
+	sess, err := NewSession("foo:443", dummyConf, "", cs.lvls, cs.log)
+	c.Assert(err, IsNil)
+	c.Check(sess.getCookie(), Equals, "")
+	sess.setCookie("COOKIE")
+	c.Check(sess.getCookie(), Equals, "COOKIE")
+	sess.ClearCookie()
+	c.Check(sess.getCookie(), Equals, "")
+}

=== modified file 'debian/changelog'
--- debian/changelog	2014-09-10 18:03:25 +0000
+++ debian/changelog	2014-10-24 14:05:51 +0000
@@ -1,3 +1,18 @@
+ubuntu-push (0.65) UNRELEASED; urgency=medium
+
+  [ John R. Lenton ]
+  * Make the common service Start(..) method take init, so we can listen for methods once everything is set up and not before.
+
+  [ Roberto Alsina ]
+  * Documentation fixes
+
+  [ Guillermo Gonzalez ]
+  * Fix click hook for legacy apps
+  * Add ClearCookie method to the session and call it from handleAccountsChange.
+  * click.AppId.SymbolicIcon() now tries to use X-Ubuntu-SymbolicIcon and then fallback to icon+"-symbolic"
+
+ -- Roberto Alsina <ralsina@yoga>  Fri, 24 Oct 2014 11:01:56 -0300
+
 ubuntu-push (0.64.1+14.10.20140910-0ubuntu1) utopic; urgency=medium
 
   [ Roberto Alsina ]

=== added file 'docs/Makefile'
--- docs/Makefile	1970-01-01 00:00:00 +0000
+++ docs/Makefile	2014-09-25 16:48:18 +0000
@@ -0,0 +1,4 @@
+all: *txt *svg
+	rst2html --link-stylesheet highlevel.txt highlevel.html
+	rst2html --link-stylesheet lowlevel.txt lowlevel.html
+

=== modified file 'docs/_common.txt'
--- docs/_common.txt	2014-09-05 14:49:44 +0000
+++ docs/_common.txt	2014-10-09 16:06:52 +0000
@@ -39,9 +39,10 @@
 
 Here, we created a helloHelper entry in hooks that has an apparmor profile and an additional JSON file for the push-helper hook.
 
-helloHelper-apparmor.json must contain **only** the push-notification-client policy group::
+helloHelper-apparmor.json must contain **only** the push-notification-client policy group and the ubuntu-push-helper template::
 
     {
+        "template": "ubuntu-push-helper",
         "policy_groups": [
             "push-notification-client"
         ],

=== modified file 'docs/example-client/components/ChatClient.qml'
--- docs/example-client/components/ChatClient.qml	2014-09-05 14:40:39 +0000
+++ docs/example-client/components/ChatClient.qml	2014-09-16 18:43:45 +0000
@@ -60,8 +60,8 @@
         if (options["enabled"]) {
             data["data"]["notification"] = {
                 "card": {
-                    "summary": nick + " says: " + message["message"],
-                    "body": "",
+                    "summary": nick + " says:",
+                    "body": message["message"],
                     "popup": options["popup"],
                     "persist": options["persist"],
                     "actions": ["appid://com.ubuntu.developer.ralsina.hello/hello/current-user-version"]

=== modified file 'docs/example-client/helloHelper-apparmor.json'
--- docs/example-client/helloHelper-apparmor.json	2014-09-05 14:40:39 +0000
+++ docs/example-client/helloHelper-apparmor.json	2014-10-09 16:06:52 +0000
@@ -1,4 +1,5 @@
 {
+    "template": "ubuntu-push-helper",
     "policy_groups": [
         "push-notification-client"
     ],

=== modified file 'docs/example-server/app.js'
--- docs/example-server/app.js	2014-09-05 14:57:17 +0000
+++ docs/example-server/app.js	2014-09-16 19:22:07 +0000
@@ -182,22 +182,40 @@
     */
     if (cfg.play_notify_form) {
         app.post("/play-notify-form", function(req, resp) {
-            resp.type('text/plain')
-            if (!req.body.data||!req.body.nick) {
-                resp.send(400, "invalid/empty fields\n")
-                return
-            }
-            var data
-            try {
-                data = JSON.parse(req.body.data)
-            } catch(e) {
-                resp.send(400, "data is not JSON\n")
-                return
+            if (!req.body.message||!req.body.nick) {
+                resp.redirect("/?error=invalid or empty fields in form")
+                return
+            }
+            var data = {
+                "message": {
+                    "from": "website",
+                    "message": req.body.message,
+                    "to": req.body.nick.toLowerCase()
+                },
+                "notification": {
+                }
+            }
+
+            if (req.body.enable) {
+                var card = {
+                    "summary": "The website says:",
+                    "body": req.body.message,
+                    "actions": ["appid://com.ubuntu.developer.ralsina.hello/hello/current-user-version"]
+                }
+                if (req.body.popup) {card["popup"] = true}
+                if (req.body.persist) {card["persist"] = true}
+                data["notification"]["card"] = card
+                if (req.body.sound) {data["notification"]["sound"] = true}
+                if (req.body.vibrate) {data["notification"]["vibrate"] = {"duration": 200}}
+                if (req.body.counter) {data["notification"]["emblem-counter"] = {
+                    "count": Math.floor(req.body.counter),
+                    "visible": true
+                }}
             }
             doNotify(true, req.body.nick, data, function() {
-                resp.send(200, 'OK\n')
+                resp.redirect("/")
             }, function() { // not found
-                resp.send(400, "unknown nick\n")
+                resp.redirect("/?error=unknown nick")
             }, resp)
         })
     }

=== modified file 'docs/example-server/config/config.js'
--- docs/example-server/config/config.js	2014-09-05 14:57:17 +0000
+++ docs/example-server/config/config.js	2014-09-16 17:16:45 +0000
@@ -11,5 +11,5 @@
     ,"happy_retry_secs": 5
     ,"expire_mins": 120
     ,"no_inbox": true
-    ,"play_notify_form": false
+    ,"play_notify_form": true
 }

=== modified file 'docs/example-server/index.html'
--- docs/example-server/index.html	2014-09-05 14:57:17 +0000
+++ docs/example-server/index.html	2014-09-16 17:16:45 +0000
@@ -1,6 +1,8 @@
 <!doctype html>
 <html>
 <head>
+<meta name="viewport" content="width=device-width, initial-scale=1">
+<link href="//maxcdn.bootstrapcdn.com/bootswatch/3.2.0/united/bootstrap.min.css" rel="stylesheet">
 <title>pushAppServer</title>
 <meta http-equiv="content-type" content="text/html; charset=UTF-8">
 </head>

=== modified file 'docs/example-server/notify-form.html'
--- docs/example-server/notify-form.html	2014-09-05 14:57:17 +0000
+++ docs/example-server/notify-form.html	2014-09-18 13:10:07 +0000
@@ -1,14 +1,69 @@
 <!doctype html>
-<html>
+<html lang="en">
 <head>
-<title>notify</title>
+<meta name="viewport" content="width=device-width, initial-scale=1">
+<link href="//maxcdn.bootstrapcdn.com/bootswatch/3.2.0/united/bootstrap.min.css" rel="stylesheet">
+<script src="https://ajax.googleapis.com/ajax/libs/jquery/1.11.1/jquery.min.js"></script>
+<script src="//maxcdn.bootstrapcdn.com/bootstrap/3.2.0/js/bootstrap.min.js"></script>
+<script>
+function enableControls() {
+    disabled = !($('#enable')[0].checked);
+    $('#popup')[0].disabled=disabled;
+    $('#persist')[0].disabled=disabled;
+    $('#sound')[0].disabled=disabled;
+    $('#vibrate')[0].disabled=disabled;
+    $('#counter')[0].disabled=disabled;
+};
+
+function showError()
+{
+    var vars = {};
+    var hashes = window.location.href.slice(window.location.href.indexOf('?') + 1).split('&');
+    for(var i = 0; i < hashes.length; i++)
+    {
+        hash = hashes[i].split('=');
+        vars[hash[0]] = decodeURIComponent(hash[1]);
+    }
+    if (vars["error"]) {
+        $("<div class='alert alert-danger' role='alert'>" + vars["error"] + "</div>").insertAfter ($("h1"))
+    }
+}
+
+</script>
+<title>Send a notification</title>
 <meta http-equiv="content-type" content="text/html; charset=UTF-8">
 </head>
 <body>
-<form method="POST" action="/play-notify-form" >
-<label for="nick">Nick:</label><br><input id="nick" name="nick" type="text" autofocus><br>
-<label for="data">Message (JSON):</label><br><textarea id="data" name="data" cols="64" rows="8"></textarea><br>
-<input type="submit" value="notify">
+<div class="container">
+<H1>Send a notification</H1>
+<form method="POST" action="/play-notify-form" role="form">
+<fieldset>
+<div class="form-group">
+    <label for="nick">Nick:</label>
+    <input  class="form-control" placeholder="Message destination" id="nick" name="nick" type="text" autofocus>
+</div>
+<div class="form-group">
+    <label for="message">Message:</label>
+    <textarea class="form-control" placeholder="Message" id="message" name="message" rows="5"></textarea>
+</div>
+<div class="checkbox">
+    <label> <input type="checkbox" id="enable" name="enable" onclick='enableControls()' checked=true> Enable Notifications </label>
+</div>
+<div class="checkbox">
+    <label> <input type="checkbox" id="popup" name="popup" checked=true> Enable Popup </label>
+    <label> <input type="checkbox" id="persist" name="persist" checked=true> Persistent </label>
+    <label> <input type="checkbox" id="sound" name="sound" checked=true> Make Sound </label>
+    <label> <input type="checkbox" id="vibrate" name="vibrate" checked=true> Vibrate </label>
+    <label for="data">
+    <input class="form-control" placeholder="Set Counter" id="counter" name="counter" type="number" value=42 autofocus>
+    </label>
+</div>
+    <button type="submit" class="btn btn-primary" value="notify">Submit</button>
+</fieldset>
 </form>
+</div>
+<script>
+    showError();
+</script>
 </body>
 </html>

=== modified file 'docs/example-server/test/app_test.js'
--- docs/example-server/test/app_test.js	2014-09-05 14:57:17 +0000
+++ docs/example-server/test/app_test.js	2014-09-16 19:57:39 +0000
@@ -393,7 +393,7 @@
     test('root-form', function(done) {
         request(this.app)
             .get('/')
-            .expect(new RegExp('<form.*action="' + PLAY_NOTIFY_FORM + '"(.|\n)*<input id="nick" name="nick"'))
+            .expect(new RegExp('<form.*action="' + PLAY_NOTIFY_FORM + '"(.|\n)*<input  class="form-control" placeholder="Message destination" id="nick" name="nick" type="text"'))
             .expect('Content-Type', 'text/html; charset=UTF-8')
             .expect(200, done)
     })
@@ -410,37 +410,20 @@
         request(this.app)
             .post(PLAY_NOTIFY_FORM)
             .type('form')
-            .send({nick: "N", data: '{"m": 1}'})
+            .send({nick: "N", message: 'foo'})
             .expect('Content-Type', 'text/plain; charset=utf-8')
-            .expect("OK\n")
-            .expect(200, function(err) {
+            .expect('Moved Temporarily. Redirecting to /')
+            .expect(302, function(err) {
                 assert.deepEqual(notify.slice(0, 2), ["N", "T"])
                 var data = notify[2]
                 assert.equal(typeof(data._ephemeral), "number")
                 assert.ok(data._ephemeral >= start)
                 delete data._ephemeral
-                assert.deepEqual(data, {"m": 1})
+                assert.deepEqual(data, {"message":{"from":"website","message":"foo","to":"n"},"notification":{}})
                 done(err)
             })
     })
 
-    test('play-notify-form-not-json', function(done) {
-        var notify
-        this.reg.findToken = function(nick, foundCb, notFoundCb, errCb) {
-            foundCb("T")
-        }
-        this.notifier.notify = function(nick, token, data) {
-            notify = [nick, token, data]
-        }
-        request(this.app)
-            .post(PLAY_NOTIFY_FORM)
-            .type('form')
-            .send({nick: "N", data: '{X'})
-            .expect('Content-Type', 'text/plain; charset=utf-8')
-            .expect("data is not JSON\n")
-            .expect(400, done)
-    })
-
     test('play-notify-form-unknown-nick', function(done) {
         this.reg.findToken = function(nick, foundCb, notFoundCb, errCb) {
             notFoundCb()
@@ -448,10 +431,10 @@
         request(this.app)
             .post(PLAY_NOTIFY_FORM)
             .set('Content-Type', 'application/json')
-            .send({nick: "N", data: '{"m": 1}'})
+            .send({nick: "N", message: 'foo'})
             .expect('Content-Type', 'text/plain; charset=utf-8')
-            .expect("unknown nick\n")
-            .expect(400, done)
+            .expect("Moved Temporarily. Redirecting to /?error=unknown%20nick")
+            .expect(302, done)
     })
 
     test('play-notify-form-unavailable', function(done) {
@@ -461,9 +444,8 @@
         request(this.app)
             .post(PLAY_NOTIFY_FORM)
             .type('form')
-            .send({nick: "N", data: '{"m": 1}'})
-            .expect('Content-Type', 'text/plain; charset=utf-8')
-            .expect('db is hopefully only momentarily :(\n')
+            .send({nick: "N", message: 'foo'})
+            .expect('{"error":"unavailable"}')
             .expect(503, function(err) {
                 done(err)
             })
@@ -476,10 +458,10 @@
         request(this.app)
             .post(PLAY_NOTIFY_FORM)
             .set('Content-Type', 'application/json')
-            .send({nick: "", data: '{"m": 1}'})
+            .send({nick: "", message: 'foo'})
             .expect('Content-Type', 'text/plain; charset=utf-8')
-            .expect("invalid/empty fields\n")
-            .expect(400, done)
+            .expect('Moved Temporarily. Redirecting to /?error=invalid%20or%20empty%20fields%20in%20form')
+            .expect(302, done)
     })
 
     test('play-notify-form-broken', function(done) {
@@ -487,7 +469,121 @@
             .post(PLAY_NOTIFY_FORM)
             .type('form')
             .send("=")
-            .expect(400, done)
+            .expect('Moved Temporarily. Redirecting to /?error=invalid%20or%20empty%20fields%20in%20form')
+            .expect(302, done)
+    })
+
+    test('play-notify-form-nick-is-lowercased', function(done) {
+        var notify
+          , start = Date.now()
+        this.reg.findToken = function(nick, foundCb, notFoundCb, errCb) {
+            foundCb("T")
+        }
+        this.notifier.notify = function(nick, token, data) {
+            notify = [nick, token, data]
+        }
+        request(this.app)
+            .post(PLAY_NOTIFY_FORM)
+            .type('form')
+            .send({nick: "N", message: 'foo'})
+            .expect('Content-Type', 'text/plain; charset=utf-8')
+            .expect('Moved Temporarily. Redirecting to /')
+            .expect(302, function(err) {
+                assert.equal(notify[2]["message"]["to"], "n")
+                done(err)
+                }
+            )
+    })
+
+    test('play-notify-disabled-notifications', function(done) {
+        var notify
+          , start = Date.now()
+        this.reg.findToken = function(nick, foundCb, notFoundCb, errCb) {
+            foundCb("T")
+        }
+        this.notifier.notify = function(nick, token, data) {
+            notify = [nick, token, data]
+        }
+        request(this.app)
+            .post(PLAY_NOTIFY_FORM)
+            .type('form')
+            .send({nick: "N", message: 'foo', persist: 'on'})
+            .expect('Content-Type', 'text/plain; charset=utf-8')
+            .expect('Moved Temporarily. Redirecting to /')
+            .expect(302, function(err) {
+                assert.deepEqual(notify[2]["notification"], {})
+                done(err)
+                }
+            )
+    })
+    test('play-notify-enabled-notifications', function(done) {
+        var notify
+          , start = Date.now()
+        this.reg.findToken = function(nick, foundCb, notFoundCb, errCb) {
+            foundCb("T")
+        }
+        this.notifier.notify = function(nick, token, data) {
+            notify = [nick, token, data]
+        }
+        request(this.app)
+            .post(PLAY_NOTIFY_FORM)
+            .type('form')
+            .send({nick: "N", message: 'foo', enable: 'on', persist: 'on'})
+            .expect('Content-Type', 'text/plain; charset=utf-8')
+            .expect('Moved Temporarily. Redirecting to /')
+            .expect(302, function(err) {
+                assert.deepEqual(notify[2]["notification"], {
+                    card: {
+                        summary: 'The website says:',
+                        body: 'foo',
+                        actions: [ 'appid://com.ubuntu.developer.ralsina.hello/hello/current-user-version' ],
+                        persist: true
+                    }})
+                done(err)
+                }
+            )
+    })
+
+    test('play-notify-enabled-all-notifications', function(done) {
+        var notify
+          , start = Date.now()
+        this.reg.findToken = function(nick, foundCb, notFoundCb, errCb) {
+            foundCb("T")
+        }
+        this.notifier.notify = function(nick, token, data) {
+            notify = [nick, token, data]
+        }
+        request(this.app)
+            .post(PLAY_NOTIFY_FORM)
+            .type('form')
+            .send({
+                nick: "N",
+                message: 'foo',
+                enable: 'on',
+                popup: 'on',
+                persist: 'on',
+                sound: 'on',
+                vibrate: 'on',
+                counter: 42
+            })
+            .expect('Content-Type', 'text/plain; charset=utf-8')
+            .expect('Moved Temporarily. Redirecting to /')
+            .expect(302, function(err) {
+                assert.deepEqual(notify[2]["notification"], {
+                    card: {
+                        summary: 'The website says:',
+                        body: 'foo',
+                        actions: [ 'appid://com.ubuntu.developer.ralsina.hello/hello/current-user-version' ],
+                        popup: true,
+                        persist: true
+                    },
+                    sound: true,
+                    vibrate: { duration: 200 },
+                    'emblem-counter': { count: 42, visible: true}
+                })
+                done(err)
+                }
+            )
     })
 
 })

=== modified file 'docs/highlevel.txt'
--- docs/highlevel.txt	2014-09-05 14:49:44 +0000
+++ docs/highlevel.txt	2014-09-25 16:48:18 +0000
@@ -1,5 +1,5 @@
 Ubuntu Push Client High Level Developer Guide
-============================================
+=============================================
 
 :Version: 0.50+
 
@@ -82,4 +82,4 @@
 
 The ``count`` property sets the counter in the application's icon to the given value.
 
-.. include:: _common.txt
\ No newline at end of file
+.. include:: _common.txt

=== modified file 'docs/lowlevel.txt'
--- docs/lowlevel.txt	2014-09-05 14:49:44 +0000
+++ docs/lowlevel.txt	2014-10-15 16:34:25 +0000
@@ -118,7 +118,7 @@
 
 
 The arguments for the Post method are APP_ID (in the example, com.ubuntu.music_music) and a JSON string
-`describing a push message. <#push-message-format>`__
+`describing a push message. <#helper-output-format>`__
 
 Depending on the contents of the push message it may trigger user-facing notifications, and will queue a
 message for the app to get via the `PopAll <#com-ubuntu-postal-popalls>`__ method.
@@ -178,4 +178,4 @@
 Set the counter to the given values.
 
 
-.. include:: _common.txt
\ No newline at end of file
+.. include:: _common.txt

=== modified file 'scripts/click-hook'
--- scripts/click-hook	2014-08-04 15:37:01 +0000
+++ scripts/click-hook	2014-10-15 11:56:43 +0000
@@ -1,4 +1,5 @@
 #!/usr/bin/python3
+# -*- python -*-
 """Collect helpers hook data into a single json file"""
 
 import argparse
@@ -24,25 +25,30 @@
 
 
 def cleanup_settings():
+    settings = Gio.Settings.new('com.ubuntu.notifications.hub')
+    blacklist = settings.get_value('blacklist').unpack()
+    if not blacklist:
+        return
+
     clickdb = Click.DB.new()
     clickdb.read()
 
-    pkgnames = []
+    pkgnames = set()
     for package in clickdb.get_packages(False):
-        pkgnames.append(package.get_property('package'))
+        pkgnames.add(package.get_property('package'))
 
-    settings = Gio.Settings.new('com.ubuntu.notifications.hub')
     goodapps = GLib.VariantBuilder.new(GLib.VariantType.new("a(ss)"))
 
-    for appname in settings.get_value('blacklist').unpack():
-        if appname[0] in pkgnames:
+    dirty = False
+
+    for appname in blacklist:
+        if appname[0] not in pkgnames and Gio.DesktopAppInfo.new(appname[1] + ".desktop") is None:
+            dirty = True
+        else:
             goodapps.add_value(tup2variant(appname))
-        elif (appname[0] == appname[1]):
-            appinfo = Gio.DesktopAppInfo.new(appname[0] + ".desktop")
-            if not appinfo is None:
-                goodapps.add_value(tup2variant(appname))
 
-    settings.set_value('blacklist', goodapps.end())
+    if dirty:
+        settings.set_value('blacklist', goodapps.end())
 
 
 def collect_helpers(helpers_data_path, helpers_data_path_tmp, hooks_path):

=== modified file 'server/broker/broker.go'
--- server/broker/broker.go	2014-05-15 13:37:22 +0000
+++ server/broker/broker.go	2014-09-25 11:16:38 +0000
@@ -26,11 +26,16 @@
 	"launchpad.net/ubuntu-push/server/store"
 )
 
+type SessionTracker interface {
+	// SessionId
+	SessionId() string
+}
+
 // Broker is responsible for registring sessions and delivering messages
 // through them.
 type Broker interface {
 	// Register the session.
-	Register(connMsg *protocol.ConnectMsg, sessionId string) (BrokerSession, error)
+	Register(connMsg *protocol.ConnectMsg, track SessionTracker) (BrokerSession, error)
 	// Unregister the session.
 	Unregister(BrokerSession)
 }

=== modified file 'server/broker/simple/simple.go'
--- server/broker/simple/simple.go	2014-05-15 15:01:50 +0000
+++ server/broker/simple/simple.go	2014-09-25 11:16:38 +0000
@@ -160,7 +160,7 @@
 
 // Register registers a session with the broker. It feeds the session
 // pending notifications as well.
-func (b *SimpleBroker) Register(connect *protocol.ConnectMsg, sessionId string) (broker.BrokerSession, error) {
+func (b *SimpleBroker) Register(connect *protocol.ConnectMsg, track broker.SessionTracker) (broker.BrokerSession, error) {
 	// xxx sanity check DeviceId
 	model, err := broker.GetInfoString(connect, "device", "?")
 	if err != nil {

=== modified file 'server/broker/simple/suite_test.go'
--- server/broker/simple/suite_test.go	2014-05-01 18:56:07 +0000
+++ server/broker/simple/suite_test.go	2014-09-25 11:16:38 +0000
@@ -32,10 +32,20 @@
 	testsuite.CommonBrokerSuite
 }
 
+// trivial session tracker
+type testTracker string
+
+func (t testTracker) SessionId() string {
+	return string(t)
+}
+
 var _ = Suite(&commonBrokerSuite{testsuite.CommonBrokerSuite{
 	MakeBroker: func(sto store.PendingStore, cfg broker.BrokerConfig, log logger.Logger) testsuite.FullBroker {
 		return NewSimpleBroker(sto, cfg, log)
 	},
+	MakeTracker: func(sessionId string) broker.SessionTracker {
+		return testTracker(sessionId)
+	},
 	RevealSession: func(b broker.Broker, deviceId string) broker.BrokerSession {
 		return b.(*SimpleBroker).registry[deviceId]
 	},

=== modified file 'server/broker/testsuite/suite.go'
--- server/broker/testsuite/suite.go	2014-07-09 14:13:47 +0000
+++ server/broker/testsuite/suite.go	2014-09-25 11:16:38 +0000
@@ -46,6 +46,8 @@
 type CommonBrokerSuite struct {
 	// Build the broker for testing.
 	MakeBroker func(store.PendingStore, broker.BrokerConfig, logger.Logger) FullBroker
+	// Build a session tracker for testing.
+	MakeTracker func(sessionId string) broker.SessionTracker
 	// Let us get to a session under the broker.
 	RevealSession func(broker.Broker, string) broker.BrokerSession
 	// Let us get to a broker.BroadcastExchange from an Exchange.
@@ -91,7 +93,7 @@
 			"device":  "model",
 			"channel": "daily",
 		},
-	}, "s1")
+	}, s.MakeTracker("s1"))
 	c.Assert(err, IsNil)
 	c.Assert(s.RevealSession(b, "dev-1"), Equals, sess)
 	c.Assert(sess.DeviceIdentifier(), Equals, "dev-1")
@@ -103,7 +105,7 @@
 	}))
 	b.Unregister(sess)
 	// just to make sure the unregister was processed
-	_, err = b.Register(&protocol.ConnectMsg{Type: "connect", DeviceId: ""}, "s2")
+	_, err = b.Register(&protocol.ConnectMsg{Type: "connect", DeviceId: ""}, s.MakeTracker("s2"))
 	c.Assert(err, IsNil)
 	c.Check(s.RevealSession(b, "dev-1"), IsNil)
 }
@@ -113,7 +115,7 @@
 	b := s.MakeBroker(sto, testBrokerConfig, nil)
 	b.Start()
 	defer b.Stop()
-	_, err := b.Register(&protocol.ConnectMsg{Type: "connect", DeviceId: "dev-1", Levels: map[string]int64{"z": 5}}, "s1")
+	_, err := b.Register(&protocol.ConnectMsg{Type: "connect", DeviceId: "dev-1", Levels: map[string]int64{"z": 5}}, s.MakeTracker("s1"))
 	c.Check(err, FitsTypeOf, &broker.ErrAbort{})
 }
 
@@ -125,11 +127,11 @@
 	info := map[string]interface{}{
 		"device": -1,
 	}
-	_, err := b.Register(&protocol.ConnectMsg{Type: "connect", Info: info}, "s1")
+	_, err := b.Register(&protocol.ConnectMsg{Type: "connect", Info: info}, s.MakeTracker("s1"))
 	c.Check(err, Equals, broker.ErrUnexpectedValue)
 	info["device"] = "m"
 	info["channel"] = -1
-	_, err = b.Register(&protocol.ConnectMsg{Type: "connect", Info: info}, "s2")
+	_, err = b.Register(&protocol.ConnectMsg{Type: "connect", Info: info}, s.MakeTracker("s2"))
 	c.Check(err, Equals, broker.ErrUnexpectedValue)
 }
 
@@ -141,7 +143,7 @@
 	b := s.MakeBroker(sto, testBrokerConfig, nil)
 	b.Start()
 	defer b.Stop()
-	sess, err := b.Register(&protocol.ConnectMsg{Type: "connect", DeviceId: "dev-1"}, "s1")
+	sess, err := b.Register(&protocol.ConnectMsg{Type: "connect", DeviceId: "dev-1"}, s.MakeTracker("s1"))
 	c.Assert(err, IsNil)
 	c.Check(len(sess.SessionChannel()), Equals, 2)
 }
@@ -151,7 +153,7 @@
 	b := s.MakeBroker(sto, testBrokerConfig, s.testlog)
 	b.Start()
 	defer b.Stop()
-	_, err := b.Register(&protocol.ConnectMsg{Type: "connect", DeviceId: "dev-1"}, "s1")
+	_, err := b.Register(&protocol.ConnectMsg{Type: "connect", DeviceId: "dev-1"}, s.MakeTracker("s1"))
 	c.Assert(err, IsNil)
 	// but
 	c.Check(s.testlog.Captured(), Matches, "ERROR unsuccessful, get channel snapshot for 0 \\(cachedOk=true\\): get channel snapshot fail\n")
@@ -167,10 +169,10 @@
 	b := s.MakeBroker(sto, testBrokerConfig, nil)
 	b.Start()
 	defer b.Stop()
-	sess1, err := b.Register(&protocol.ConnectMsg{Type: "connect", DeviceId: "dev-1"}, "s1")
+	sess1, err := b.Register(&protocol.ConnectMsg{Type: "connect", DeviceId: "dev-1"}, s.MakeTracker("s1"))
 	c.Assert(err, IsNil)
 	clearOfPending(c, sess1)
-	sess2, err := b.Register(&protocol.ConnectMsg{Type: "connect", DeviceId: "dev-1"}, "s2")
+	sess2, err := b.Register(&protocol.ConnectMsg{Type: "connect", DeviceId: "dev-1"}, s.MakeTracker("s2"))
 	c.Assert(err, IsNil)
 	// previous session got signaled by sending nil on its channel
 	var sentinel broker.Exchange
@@ -186,7 +188,7 @@
 	c.Assert(s.RevealSession(b, "dev-1"), Equals, sess2)
 	b.Unregister(sess1)
 	// just to make sure the unregister was processed
-	_, err = b.Register(&protocol.ConnectMsg{Type: "connect", DeviceId: ""}, "s3")
+	_, err = b.Register(&protocol.ConnectMsg{Type: "connect", DeviceId: ""}, s.MakeTracker("s3"))
 	c.Assert(err, IsNil)
 	c.Check(s.RevealSession(b, "dev-1"), Equals, sess2)
 }
@@ -198,10 +200,10 @@
 	b := s.MakeBroker(sto, testBrokerConfig, nil)
 	b.Start()
 	defer b.Stop()
-	sess1, err := b.Register(&protocol.ConnectMsg{Type: "connect", DeviceId: "dev-1"}, "s1")
+	sess1, err := b.Register(&protocol.ConnectMsg{Type: "connect", DeviceId: "dev-1"}, s.MakeTracker("s1"))
 	c.Assert(err, IsNil)
 	clearOfPending(c, sess1)
-	sess2, err := b.Register(&protocol.ConnectMsg{Type: "connect", DeviceId: "dev-2"}, "s2")
+	sess2, err := b.Register(&protocol.ConnectMsg{Type: "connect", DeviceId: "dev-2"}, s.MakeTracker("s2"))
 	c.Assert(err, IsNil)
 	clearOfPending(c, sess2)
 	// add notification to channel *after* the registrations
@@ -258,7 +260,7 @@
 	b := s.MakeBroker(sto, testBrokerConfig, s.testlog)
 	b.Start()
 	defer b.Stop()
-	sess, err := b.Register(&protocol.ConnectMsg{Type: "connect", DeviceId: "dev-1"}, "s1")
+	sess, err := b.Register(&protocol.ConnectMsg{Type: "connect", DeviceId: "dev-1"}, s.MakeTracker("s1"))
 	c.Assert(err, IsNil)
 	clearOfPending(c, sess)
 	b.Broadcast(store.SystemInternalChannelId)
@@ -279,10 +281,10 @@
 	b := s.MakeBroker(sto, testBrokerConfig, nil)
 	b.Start()
 	defer b.Stop()
-	sess1, err := b.Register(&protocol.ConnectMsg{Type: "connect", DeviceId: "dev1"}, "s1")
+	sess1, err := b.Register(&protocol.ConnectMsg{Type: "connect", DeviceId: "dev1"}, s.MakeTracker("s1"))
 	c.Assert(err, IsNil)
 	clearOfPending(c, sess1)
-	sess2, err := b.Register(&protocol.ConnectMsg{Type: "connect", DeviceId: "dev2"}, "s2")
+	sess2, err := b.Register(&protocol.ConnectMsg{Type: "connect", DeviceId: "dev2"}, s.MakeTracker("s2"))
 	c.Assert(err, IsNil)
 	clearOfPending(c, sess2)
 	// add notification to channel *after* the registrations
@@ -313,7 +315,7 @@
 	b := s.MakeBroker(sto, testBrokerConfig, nil)
 	b.Start()
 	defer b.Stop()
-	sess1, err := b.Register(&protocol.ConnectMsg{Type: "connect", DeviceId: "dev3"}, "s1")
+	sess1, err := b.Register(&protocol.ConnectMsg{Type: "connect", DeviceId: "dev3"}, s.MakeTracker("s1"))
 	c.Assert(err, IsNil)
 	muchLater := store.Metadata{Expiration: time.Now().Add(10 * time.Minute)}
 	sto.AppendToUnicastChannel(chanId1, "app1", notification1, "msg1", muchLater)
@@ -338,7 +340,7 @@
 	b := s.MakeBroker(sto, testBrokerConfig, s.testlog)
 	b.Start()
 	defer b.Stop()
-	sess1, err := b.Register(&protocol.ConnectMsg{Type: "connect", DeviceId: "dev3"}, "s1")
+	sess1, err := b.Register(&protocol.ConnectMsg{Type: "connect", DeviceId: "dev3"}, s.MakeTracker("s1"))
 	c.Assert(err, IsNil)
 	_, _, err = sess1.Get(chanId1, false)
 	c.Assert(err, ErrorMatches, "get channel snapshot fail")
@@ -355,7 +357,7 @@
 	b := s.MakeBroker(sto, testBrokerConfig, nil)
 	b.Start()
 	defer b.Stop()
-	sess1, err := b.Register(&protocol.ConnectMsg{Type: "connect", DeviceId: "dev3"}, "s1")
+	sess1, err := b.Register(&protocol.ConnectMsg{Type: "connect", DeviceId: "dev3"}, s.MakeTracker("s1"))
 	c.Assert(err, IsNil)
 	clearOfPending(c, sess1)
 	bcast := &broker.BroadcastExchange{ChanId: store.SystemInternalChannelId, TopLevel: 99}

=== modified file 'server/dev/server.go'
--- server/dev/server.go	2014-09-01 16:16:01 +0000
+++ server/dev/server.go	2014-09-25 11:20:00 +0000
@@ -29,6 +29,7 @@
 	"launchpad.net/ubuntu-push/server"
 	"launchpad.net/ubuntu-push/server/api"
 	"launchpad.net/ubuntu-push/server/broker/simple"
+	"launchpad.net/ubuntu-push/server/listener"
 	"launchpad.net/ubuntu-push/server/session"
 	"launchpad.net/ubuntu-push/server/store"
 )
@@ -97,8 +98,9 @@
 	handler := api.PanicTo500Handler(mux, logger)
 	go server.HTTPServeRunner(nil, handler, &cfg.HTTPServeParsedConfig, nil)()
 	// listen for device connections
+	resource := &listener.NopSessionResourceManager{}
 	server.DevicesRunner(lst, func(conn net.Conn) error {
 		track := session.NewTracker(logger)
 		return session.Session(conn, broker, cfg, track)
-	}, logger, &cfg.DevicesParsedConfig)()
+	}, logger, resource, &cfg.DevicesParsedConfig)()
 }

=== modified file 'server/listener/listener.go'
--- server/listener/listener.go	2014-09-04 12:51:08 +0000
+++ server/listener/listener.go	2014-09-25 11:20:00 +0000
@@ -67,10 +67,20 @@
 	return false
 }
 
+// SessionResourceManager allows to limit resource usage tracking connections.
+type SessionResourceManager interface {
+	ConsumeConn()
+}
+
+// NOP SessionResourceManager.
+type NopSessionResourceManager struct{}
+
+func (r *NopSessionResourceManager) ConsumeConn() {}
+
 // AcceptLoop accepts connections and starts sessions for them.
-func (dl *DeviceListener) AcceptLoop(session func(net.Conn) error, logger logger.Logger) error {
+func (dl *DeviceListener) AcceptLoop(session func(net.Conn) error, resource SessionResourceManager, logger logger.Logger) error {
 	for {
-		// xxx enforce a connection limit
+		resource.ConsumeConn()
 		conn, err := dl.Listener.Accept()
 		if err != nil {
 			if handleTemporary(err) {

=== modified file 'server/listener/listener_test.go'
--- server/listener/listener_test.go	2014-09-04 16:41:39 +0000
+++ server/listener/listener_test.go	2014-09-25 11:20:00 +0000
@@ -142,21 +142,44 @@
 	c.Check(buf[0], Equals, byte(expected))
 }
 
+type testSessionResourceManager struct {
+	event chan string
+}
+
+func (r *testSessionResourceManager) ConsumeConn() {
+	r.event <- "consume"
+}
+
+// takeNext takes a string from given channel with a 5s timeout
+func takeNext(ch <-chan string) string {
+	select {
+	case <-time.After(5 * time.Second):
+		panic("test protocol exchange stuck: too long waiting")
+	case v := <-ch:
+		return v
+	}
+}
+
 func (s *listenerSuite) TestDeviceAcceptLoop(c *C) {
 	lst, err := DeviceListen(nil, &testDevListenerCfg{"127.0.0.1:0"})
 	c.Check(err, IsNil)
 	defer lst.Close()
 	errCh := make(chan error)
+	rEvent := make(chan string)
+	resource := &testSessionResourceManager{rEvent}
 	go func() {
-		errCh <- lst.AcceptLoop(testSession, s.testlog)
+		errCh <- lst.AcceptLoop(testSession, resource, s.testlog)
 	}()
 	listenerAddr := lst.Addr().String()
+	c.Check(takeNext(rEvent), Equals, "consume")
 	conn1, err := testTlsDial(listenerAddr)
 	c.Assert(err, IsNil)
+	c.Check(takeNext(rEvent), Equals, "consume")
 	defer conn1.Close()
 	testWriteByte(c, conn1, '1')
 	conn2, err := testTlsDial(listenerAddr)
 	c.Assert(err, IsNil)
+	c.Check(takeNext(rEvent), Equals, "consume")
 	defer conn2.Close()
 	testWriteByte(c, conn2, '2')
 	testReadByte(c, conn1, '1')
@@ -186,8 +209,9 @@
 	c.Check(err, IsNil)
 	defer lst.Close()
 	errCh := make(chan error)
+	resource := &NopSessionResourceManager{}
 	go func() {
-		errCh <- lst.AcceptLoop(testSession, s.testlog)
+		errCh <- lst.AcceptLoop(testSession, resource, s.testlog)
 	}()
 	listenerAddr := lst.Addr().String()
 	connectMany := helpers.ScriptAbsPath("connect-many.py")
@@ -210,11 +234,12 @@
 	c.Check(err, IsNil)
 	defer lst.Close()
 	errCh := make(chan error)
+	resource := &NopSessionResourceManager{}
 	go func() {
 		errCh <- lst.AcceptLoop(func(conn net.Conn) error {
 			defer conn.Close()
 			panic("session crash")
-		}, s.testlog)
+		}, resource, s.testlog)
 	}()
 	listenerAddr := lst.Addr().String()
 	_, err = testTlsDial(listenerAddr)
@@ -231,8 +256,9 @@
 	c.Check(err, IsNil)
 	defer lst.Close()
 	errCh := make(chan error)
+	resource := &NopSessionResourceManager{}
 	go func() {
-		errCh <- lst.AcceptLoop(testSession, s.testlog)
+		errCh <- lst.AcceptLoop(testSession, resource, s.testlog)
 	}()
 	listenerAddr := lst.Addr().String()
 	c.Check(listenerAddr, Equals, foreignLst.Addr().String())

=== modified file 'server/runner_devices.go'
--- server/runner_devices.go	2014-09-01 16:03:06 +0000
+++ server/runner_devices.go	2014-09-25 11:20:00 +0000
@@ -62,7 +62,7 @@
 // DevicesRunner returns a function to accept device connections.
 // If adoptLst is not nil it will be used as the underlying listener, instead
 // of creating one, wrapped in a TLS layer.
-func DevicesRunner(adoptLst net.Listener, session func(net.Conn) error, logger logger.Logger, parsedCfg *DevicesParsedConfig) func() {
+func DevicesRunner(adoptLst net.Listener, session func(net.Conn) error, logger logger.Logger, resource listener.SessionResourceManager, parsedCfg *DevicesParsedConfig) func() {
 	BootLogger.Debugf("PingInterval: %s, ExchangeTimeout %s", parsedCfg.PingInterval(), parsedCfg.ExchangeTimeout())
 	var rlim syscall.Rlimit
 	err := syscall.Getrlimit(syscall.RLIMIT_NOFILE, &rlim)
@@ -76,7 +76,7 @@
 	}
 	BootLogListener("devices", lst)
 	return func() {
-		err = lst.AcceptLoop(session, logger)
+		err = lst.AcceptLoop(session, resource, logger)
 		if err != nil {
 			BootLogFatalf("accepting device connections: %v", err)
 		}

=== modified file 'server/runner_test.go'
--- server/runner_test.go	2014-09-04 16:41:39 +0000
+++ server/runner_test.go	2014-09-25 11:20:00 +0000
@@ -27,6 +27,7 @@
 	. "launchpad.net/gocheck"
 
 	"launchpad.net/ubuntu-push/config"
+	"launchpad.net/ubuntu-push/server/listener"
 	helpers "launchpad.net/ubuntu-push/testing"
 )
 
@@ -111,6 +112,8 @@
 	},
 }
 
+var resource = &listener.NopSessionResourceManager{}
+
 func (s *runnerSuite) TestDevicesRunner(c *C) {
 	prevBootLogger := BootLogger
 	testlog := helpers.NewTestLogger(c, "debug")
@@ -118,7 +121,7 @@
 	defer func() {
 		BootLogger = prevBootLogger
 	}()
-	runner := DevicesRunner(nil, func(conn net.Conn) error { return nil }, BootLogger, &testDevicesParsedConfig)
+	runner := DevicesRunner(nil, func(conn net.Conn) error { return nil }, BootLogger, resource, &testDevicesParsedConfig)
 	c.Assert(s.lst, Not(IsNil))
 	s.lst.Close()
 	c.Check(s.kind, Equals, "devices")
@@ -135,7 +138,7 @@
 	lst0, err := net.Listen("tcp", "127.0.0.1:0")
 	c.Assert(err, IsNil)
 	defer lst0.Close()
-	DevicesRunner(lst0, func(conn net.Conn) error { return nil }, BootLogger, &testDevicesParsedConfig)
+	DevicesRunner(lst0, func(conn net.Conn) error { return nil }, BootLogger, resource, &testDevicesParsedConfig)
 	c.Assert(s.lst, Not(IsNil))
 	c.Check(s.lst.Addr().String(), Equals, lst0.Addr().String())
 	s.lst.Close()

=== modified file 'server/session/session.go'
--- server/session/session.go	2014-06-19 13:06:52 +0000
+++ server/session/session.go	2014-09-25 11:16:38 +0000
@@ -36,7 +36,7 @@
 }
 
 // sessionStart manages the start of the protocol session.
-func sessionStart(proto protocol.Protocol, brkr broker.Broker, cfg SessionConfig, sessionId string) (broker.BrokerSession, error) {
+func sessionStart(proto protocol.Protocol, brkr broker.Broker, cfg SessionConfig, track SessionTracker) (broker.BrokerSession, error) {
 	var connMsg protocol.ConnectMsg
 	proto.SetDeadline(time.Now().Add(cfg.ExchangeTimeout()))
 	err := proto.ReadMessage(&connMsg)
@@ -53,7 +53,7 @@
 	if err != nil {
 		return nil, err
 	}
-	return brkr.Register(&connMsg, sessionId)
+	return brkr.Register(&connMsg, track)
 }
 
 var errOneway = errors.New("oneway")
@@ -204,7 +204,7 @@
 		return track.End(&broker.ErrAbort{"unexpected wire format version"})
 	}
 	proto := protocol.NewProtocol0(conn)
-	sess, err := sessionStart(proto, brkr, cfg, track.SessionId())
+	sess, err := sessionStart(proto, brkr, cfg, track)
 	if err != nil {
 		return track.End(err)
 	}

=== modified file 'server/session/session_test.go'
--- server/session/session_test.go	2014-08-27 21:19:24 +0000
+++ server/session/session_test.go	2014-09-25 11:16:38 +0000
@@ -130,7 +130,8 @@
 	return &testBroker{registration: make(chan interface{}, 2)}
 }
 
-func (tb *testBroker) Register(connect *protocol.ConnectMsg, sessionId string) (broker.BrokerSession, error) {
+func (tb *testBroker) Register(connect *protocol.ConnectMsg, track broker.SessionTracker) (broker.BrokerSession, error) {
+	sessionId := track.SessionId()
 	tb.registration <- fmt.Sprintf("register %s %s", connect.DeviceId, sessionId)
 	return &testing.TestBrokerSession{DeviceId: connect.DeviceId}, tb.err
 }
@@ -148,7 +149,7 @@
 	brkr := newTestBroker()
 	go func() {
 		var err error
-		sess, err = sessionStart(tp, brkr, cfg10msPingInterval5msExchangeTout, "s1")
+		sess, err = sessionStart(tp, brkr, cfg10msPingInterval5msExchangeTout, &tracker{sessionId: "s1"})
 		errCh <- err
 	}()
 	c.Check(takeNext(down), Equals, "deadline 5ms")
@@ -175,7 +176,7 @@
 	brkr.err = errRegister
 	go func() {
 		var err error
-		sess, err = sessionStart(tp, brkr, cfg10msPingInterval5msExchangeTout, "s2")
+		sess, err = sessionStart(tp, brkr, cfg10msPingInterval5msExchangeTout, &tracker{sessionId: "s2"})
 		errCh <- err
 	}()
 	up <- protocol.ConnectMsg{Type: "connect", ClientVer: "1", DeviceId: "dev-1"}
@@ -190,7 +191,7 @@
 	down := make(chan interface{}, 5)
 	tp := &testProtocol{up, down}
 	up <- io.ErrUnexpectedEOF
-	_, err := sessionStart(tp, nil, cfg10msPingInterval5msExchangeTout, "s3")
+	_, err := sessionStart(tp, nil, cfg10msPingInterval5msExchangeTout, &tracker{sessionId: "s3"})
 	c.Check(err, Equals, io.ErrUnexpectedEOF)
 }
 
@@ -200,7 +201,7 @@
 	tp := &testProtocol{up, down}
 	up <- protocol.ConnectMsg{Type: "connect"}
 	up <- io.ErrUnexpectedEOF
-	_, err := sessionStart(tp, nil, cfg10msPingInterval5msExchangeTout, "s4")
+	_, err := sessionStart(tp, nil, cfg10msPingInterval5msExchangeTout, &tracker{sessionId: "s4"})
 	c.Check(err, Equals, io.ErrUnexpectedEOF)
 	// sanity
 	c.Check(takeNext(down), Matches, "deadline.*")
@@ -212,7 +213,7 @@
 	down := make(chan interface{}, 5)
 	tp := &testProtocol{up, down}
 	up <- protocol.ConnectMsg{Type: "what"}
-	_, err := sessionStart(tp, nil, cfg10msPingInterval5msExchangeTout, "s5")
+	_, err := sessionStart(tp, nil, cfg10msPingInterval5msExchangeTout, &tracker{sessionId: "s5"})
 	c.Check(err, DeepEquals, &broker.ErrAbort{"expected CONNECT message"})
 }
 

